from distutils.core import setup

setup(
        name='model_assertions',
        version='0.0.2',
        packages=['omg'],
        author='Daniel Kang',
        author_email='daniel.d.kang@gmail.com'
)
